import unittest

from pyramid import testing


class ViewTests(unittest.TestCase):
    def setUp(self):
        self.config = testing.setUp()

    def tearDown(self):
        testing.tearDown()

    def test_first_call_cas_login(self):
        from pyramid_cas.views import caslogin
        request = testing.DummyRequest()
        info = caslogin(request)
        ticket = request.GET.get('ticket')
        self.assertEqual(ticket, None)

    def test_cas_logout(self):
        from pyramid_cas.views import caslogout
        request = testing.DummyRequest()
        info = caslogout(request)
        self.assertEqual(info.status, '302 Found')




import logging
from urllib import urlencode, urlopen
from urlparse import urljoin
from xml.etree import ElementTree

from pyramid.httpexceptions import HTTPFound

logger = logging.getLogger(__name__)


class CASProvider(object):
    """
    Provides Jasig CAS authentication mechanism for pyramid
    """

    def getserviceurl(self, request):
        """
        Returns current application's url
        """
        if 'HTTP_X_FORWARDED_HOST' in request.headers:
            # TODO get the http from the request
            applicationurl = 'http://' + request['HTTP_X_FORWARDED_HOST']
        else:
            applicationurl = request.host_url + request.path
        return applicationurl

    def getloginurl(self, request, service):
        """
        Generates login url
        """
        params = {'service': service}
        config = request.registry.settings
        cas_server = config.get('pyramid_cas.cas_server')
        return urljoin(cas_server, 'login') + '?' + urlencode(params)

    def verifycas20(self, request, ticket, service):
        """Verifies CAS 2.0+ XML-based authentication ticket.
        Returns username on success and None on failure.
        """
        params = {'ticket': ticket, 'service': service}
        config = request.registry.settings
        cas_server = config.get('pyramid_cas.cas_server')
        url = (urljoin(cas_server, 'serviceValidate') + '?' + urlencode(params))
        page = urlopen(url)
        try:
            response = page.read()
            tree = ElementTree.fromstring(response)
            if tree[0].tag.endswith('authenticationSuccess'):
                return tree[0][0].text
            else:
                return None
        finally:
            page.close()

    def getlogouturl(self, request):
        """Generates CAS logout URL"""
        config = request.registry.settings
        cas_server = config.get('pyramid_cas.cas_server')
        url = urljoin(cas_server, 'logout?service=%s'.format(self.getserviceurl(request)))
        return url

    def sendtoservice(self, request):
        return HTTPFound(location=self.getloginurl(request, self.getserviceurl(request)))



from pyramid.exceptions import ConfigurationError


def includeme(config):
    settings = config.registry.settings
    if 'pyramid_cas.cas_server' not in settings:
        raise ConfigurationError('The CAS server has not been defined')
    if 'pyramid_cas.redirect_route' not in settings:
        raise ConfigurationError('The redirect route has not been defined')
    config.scan('pyramid_cas.views')




# python
import sys
import logging

# pyramid
from pyramid.httpexceptions import HTTPFound, HTTPForbidden
from pyramid.security import remember, forget, unauthenticated_userid
from pyramid.view import view_config


## pyramid_cs
from pyramid_cas.services import CASProvider
logger = logging.getLogger(__name__)

cas = CASProvider()


@view_config(name='cas-login', renderer='string')
def caslogin(request, return_url=None):
    """
    Cas login and user challenger view
    """
    service = cas.getserviceurl(request)
    username = unauthenticated_userid(request)
    if username is None:
        ticket = request.GET.get('ticket')
        if ticket is None:
            return cas.sendtoservice(request)
        username = cas.verifycas20(request, ticket, service)
        if username is None:
            return 'no user'

        settings = request.registry.settings
        if 'pyramid_cas.callback.get_user' in settings:
            callable = settings['pyramid_cas.callback.get_user']
            module = callable.split('.')[0] + '.' + callable.split('.')[1]
            caller = sys.modules[module]
            method = getattr(caller, callable.split('.')[2])
            user = method(username, request)
        else:
            user = username
        headers = remember(request, user, max_age='86400')

        # fall back to setting from config file if return_url isn't provided
        redirect_to = return_url or request.route_url(settings['pyramid_cas.redirect_route'])
        return HTTPFound(location=redirect_to, headers=headers)
    else:
        raise HTTPForbidden


@view_config(name='cas-logout', renderer='string')
def caslogout(request):
    """
    Cas logout page
    """
    headers = forget(request)
    request.session.clear()
    return HTTPFound(location=cas.getlogouturl(request), headers=headers)



import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()
CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()

requires = ['pyramid',
            'SQLAlchemy',
            'transaction',
            'pyramid_tm',
            'pyramid_debugtoolbar',
            'zope.sqlalchemy']

setup(name='pyramid_cas',
      version='0.3',
      description='A CAS client for use with the Pyramid web framework',
      long_description=README + '\n\n' + CHANGES,
      classifiers=['Programming Language :: Python',
                   'Framework :: Pyramid',
                   'Topic :: Internet :: WWW/HTTP',
                   'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
                   'License :: OSI Approved :: Apache Software License'],
      author='Ryan Fox',
      author_email='ryan@foxrow.com',
      url='https://github.com/ryanfox/pyramid_cas',
      keywords='web pyramid pylons cas authentication',
      license='Apache license',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      install_requires=requires,
      tests_require=requires,
      test_suite='pyramid_cas',
      entry_points="""\
      [paste.app_factory]
      main = pyramid_cas:main
      """,
      paster_plugins=['pyramid'])
